
/*

  0x333stalk_config.c
  
     heart of application
     here coded various function what client and server use

     [1]   TCP/IP functions for connection...
     [2]   functions that use other RSA functions... like generateKEYS, exchangeKEYS, stalk and so goes way...
     [3]   windows functions...
     [4]   some functions for catch signals... and so goes way
     [5]   ...
     
     any bugs, ideas send mail to nsn@0x333.org
  
  coded by nsn / www.0x333.org

*/

#include "config.h"

xwin                     mywin,
                         centerlinewin,
                         hiswin;

int                      MAXROWS = 0, 
                         MAXCOLS = 0;

/* 
   this function is called too when pressed CTRL+C (SIGINT INTERRUPT), used just for some cleanup... 
   in main initialize it, giving it pointer of our function... signal(SIGINT, myexit);
*/

void 
myexit(int signo)
{

   cleanwin();
   fflush(stdout);
   printf("\n[!] INTERRUPTED EXITING, CLEANING;\n");

   #ifdef CLIENT
      remove(NAMEFILEPUBKEY_CLIENT);   
      remove(NAMETEMPFILEPUBKEY_SERVER);
   #endif 
   
   #ifdef SERVER
      remove(NAMEFILEPUBKEY_SERVER);
      remove(NAMETEMPFILEPUBKEY_CLIENT);
   #endif 
   
   exit(EXITCODE);
   
}

void
error(const char *fmt, ...)
{

    char      * string = NULL;
    va_list     args;


    va_start(args, fmt);
    vasprintf(&string, fmt, args);
    va_end(args);

    if (string) {
       printf("%s; fatal aborting.\n", string);
       free(string);
    }

    myexit(EXITCODE);
}

void
writesocket(int fdsocket, int fatal, const char *fmt, ...)
{

    char      * string = NULL;
    va_list     args;

    va_start(args, fmt);
    vasprintf(&string, fmt, args);
    va_end(args);
       
    if (string) {

       write(fdsocket,string, strlen(string));
                   
       if ((fatal == CLOSESOCKET) || (fatal == ABORTALL)) {
          close(fdsocket);

          #ifdef SERVER
             FRIENDLOGGED = ISFALSE; //global var
          #endif    
          
          printf("Killing Socket reason: %s;\n", string);
       }   
       
      free(string);       
      
      if (fatal == ABORTALL)
         myexit(EXITCODE);
         
   }  

}


void
genrsa_callback (int p, int n, void *arg)
{

   char c='#';


   switch (p) {
      case 0 : c = '.'  ;break;  //test if prime
      case 1 : c = '+'  ;break;  //prime number test
      case 2 : c = '*'  ;break;  //reject  gcd(p,e)!=0
      case 3 : c = '\n' ;break;  //prime
   }

   printf("%c", c);

}   

RSA *
generateKEYS(int BITS, unsigned long E, char *OUTPATH)
{

      FILE    * pubstream;
      RSA     * key = NULL;
      

      key = RSA_new();
      key = RSA_generate_key(BITS , E, genrsa_callback, NULL);
      
      if (RSA_check_key(key)) {
      
         if (!(pubstream = fopen(OUTPATH, "w")) || !(PEM_write_RSAPublicKey(pubstream, key)))
            error("Failed while writing pubkey in %s", OUTPATH);
         
         fclose(pubstream);   
         
      }
      else   
         error("Failed verify RSA keys");
         
return key;         
}

BOOLEAN 
exchangeKeys(int socketfriend, RSA **key, char *INPATH, char *OUTPATH)
{

    FILE       * pubstream = NULL,
               * outstream = NULL;
    BOOLEAN      status = ISTRUE;
    int          len = 0;
    char       * endmsg = "-----END RSA PUBLIC KEY-----",
                 datastring[BITSENC];
    

    if (pubstream = fopen(INPATH, "r")) {
    
       while(!feof(pubstream)) {

          memset(datastring, 0, BITSENC);
          len = fread(datastring, 1, BITSENC - 1, pubstream);

          if (len > 0)
             writesocket(socketfriend, NORMALTEXT, "%s", datastring);
          else
             error("Error while reading file %s", INPATH);
             
       }
       
       fclose(pubstream);
       
       if (outstream = fopen(OUTPATH, "w+")) {

          do {
          
             memset(datastring, 0, BITSENC);
             read(socketfriend, datastring, BITSENC - 1);
             fprintf(outstream, "%s", datastring);
             
          }while(!strstr(datastring, endmsg));
          
          fflush(outstream);
          rewind(outstream);
     
          if ((*key = PEM_read_RSAPublicKey(outstream, NULL, NULL, NULL)))
             printf("*** Pubkey RECEVIED (if is incorrect target host will not able decrypt any msg)\n");
          else  
             status = ISFALSE;
                
          fclose(outstream);   

       }
       else
          error("Error while creating file %s for store pubkey of client connected", OUTPATH);    
    
    }
    else
       error("Failed read pubkey by file %s", INPATH);
    
    
return status;    
}


char *
getDateTime(char *format)
{

static char          timebuf[25];
time_t               tmptime_t;
struct tm          * tmptm;


   tmptime_t = time(NULL);
   tmptm = localtime(&tmptime_t);
   strftime(timebuf, 25, format, tmptm);
   
   return timebuf;
   
}
    
 /*
   in stalk function we don't use function writesocket
   because encrytped buffer can contains various NULL BYTES and vasprtinf and strlen can fails easly.
 */

void
stalk(int socketfriend, RSA *hispubkey, RSA *mykey, char *mynickname)
{

   unsigned char       * plaintext                        = NULL,
                       * encdata                          = NULL,
                       * hisnickname                      = NULL,
                       * timebuf                          = NULL;
   int                   flenin                           = 0, // size input for plaintext
                         flenout                          = 0, // size output for encdata
                         size                             = BITSENC / 8, //256 bytes 
                         RSA_PKCS1_OAEP_PADDING_OFFSET    = 41, //oeap padding 
                         pid                              = -1; 

   flenin         =    RSA_size(hispubkey) - RSA_PKCS1_OAEP_PADDING_OFFSET;
   plaintext      =    malloc(flenin);
   encdata        =    malloc(size);
   hisnickname    =    malloc(MAXLENNICK);

   
   memset(plaintext, 0, flenin);
   memset(encdata, 0, size);
   memset(hisnickname, 0, MAXLENNICK);

   snprintf(plaintext, flenin - 1, "NICK %s", mynickname);
 
   if (RSA_public_encrypt(strlen(plaintext), plaintext, encdata, hispubkey, RSA_PKCS1_OAEP_PADDING) > 0) {

      write(socketfriend, encdata, size);
      memset(encdata, 0, size);
      read(socketfriend, encdata, size);   
      memset(plaintext, 0, flenin);

      if (RSA_private_decrypt(size, encdata, plaintext, mykey, RSA_PKCS1_OAEP_PADDING) > 0) {

         if (!memcmp(plaintext, "NICK", strlen("NICK"))) { 
      
            snprintf(hisnickname, MAXLENNICK - 1, "%s", plaintext + strlen("NICK "));

            signal(SIGWINCH, adjust);   // catching signal window resize
            
            init_display();
            
            while (1) {
              
               if (pid == -1)
                  pid = fork();

               memset(plaintext, 0, flenin);
               memset(encdata, 0, size);
               
               if (!pid) {
               
                  read(socketfriend, encdata, size);
                  
                  if (RSA_private_decrypt(size, encdata, plaintext, mykey, RSA_PKCS1_OAEP_PADDING) > 0) {

                     //getting date and time formatted 
                     timebuf = getDateTime("%d.%m.%y/%T");
                       
                     //delete a line from hiswin.win and reset cursor
                     wmove(hiswin.win, 0, 0);
                     wdeleteln(hiswin.win);
                     wmove(hiswin.win, hiswin.nlines - 1, 0); 
                       
                     //output
                     wprintw(hiswin.win, "[%s/%s] %s", timebuf, hisnickname, plaintext);
                     wnoutrefresh(hiswin.win);
                       
                     //reset cursor on mywin
                     wmove(mywin.win, mywin.nlines - 1, 0);
                     wnoutrefresh(mywin.win);
                     doupdate();
                     
                  }     
                  else {
                  
                     wmove(mywin.win, mywin.nlines - 1, 0);
                     wprintw(mywin.win,"[FATAL]: Error while Decrypting during conversation, probably client host gone down\n");
                     wrefresh(mywin.win);
                     myexit(EXITCODE);
                     
                  }   

               }
               else {
               
                  //getting input from window called mywin
                  do {

                     wmove(mywin.win, mywin.nlines - 1, 0);
                     
                     /* 
                        this because plaintext buf is allocated with flenin size
                        but we want get only a max of chars equal to MAXCOLS
                        because MAXCOLS says us, max number of chars what user
                        can write in a single row on screen... 
                        if MAXCOLS is bigger or equal to flenin means what
                        plaintext buffer will be not able contains all data inserted... and in this
                        case then we switch MAXCOLS = flenin;
                       
                        PARANOIC TEST
                     */
                     
                     if (MAXCOLS >= flenin)
                        MAXCOLS = flenin;

                     wgetnstr(mywin.win, plaintext, MAXCOLS - 1);
                                 
                  }while(!strcmp(plaintext,""));

                  //getting date and time formatted 
                  timebuf = getDateTime("%d.%m.%y/%T");
                  
                  //output 
                  wprintw(mywin.win, "[%s/%s] %s", timebuf, mynickname, plaintext);
                  
                  //delete a line, and reset cursor
                  wmove(mywin.win, 0, 0);
                  wdeleteln(mywin.win);
                  wmove(mywin.win, mywin.nlines - 1, 0);
                  wnoutrefresh(mywin.win);
                  doupdate();
                  
                  if (RSA_public_encrypt(strlen(plaintext), plaintext, encdata, hispubkey, RSA_PKCS1_OAEP_PADDING) > 0)
                     write(socketfriend, encdata, size);
                  else
                     writesocket(socketfriend, ABORTALL, "Error while Encrypting during conversation");
                    
               }      

             }
             
            cleanwin();
            
         }
         else
            writesocket(socketfriend, CLOSESOCKET, "Protocol mismatch PROTO STEP TALK");
      }      
      else
         writesocket(socketfriend, ABORTALL, "Error while decrypting, probably client host gone down");
   }      
   else   
      writesocket(socketfriend, CLOSESOCKET, "Error while encrypting with your pubkey");
      
      
   free(plaintext);
   free(encdata);   
   free(hisnickname);   
         
}

void
cleanwin()
{

   delwin(mywin.win);
   delwin(centerlinewin.win);
   delwin(hiswin.win);
   endwin();
      
}

void
adjust(int signo)
{

   struct winsize size;

   if (!ioctl(fileno(stdout), TIOCGWINSZ, &size)) {
      resize_term(size.ws_row, size.ws_col);
	  wrefresh(curscr);
	  clear();
      init_display();
   }   

}

void
init_display()
{

   initscr();

   getmaxyx(stdscr, MAXROWS, MAXCOLS);
            
   mywin.nlines  = MAXROWS / 2;
   mywin.ncols   = MAXCOLS;
   mywin.beginy  = mywin.beginx  = 0;
   mywin.win     = newwin(mywin.nlines, mywin.ncols, mywin.beginy, mywin.beginx);
   scrollok(mywin.win, FALSE);
   clearok(mywin.win, FALSE);
   wclear(mywin.win);

   centerlinewin.nlines  = 1;
   centerlinewin.ncols   = MAXCOLS;
   centerlinewin.beginy  = mywin.nlines;
   centerlinewin.beginx  = 0;
   centerlinewin.win     = newwin(centerlinewin.nlines, centerlinewin.ncols, centerlinewin.beginy, centerlinewin.beginx);
   wclear(centerlinewin.win);
   whline(centerlinewin.win, 175, MAXCOLS);
           
   hiswin.nlines  = (MAXROWS / 2) - 1;
   hiswin.ncols   = MAXCOLS;
   hiswin.beginy  = mywin.nlines + 1;
   hiswin.beginx  = 0;
   hiswin.win     = newwin(hiswin.nlines, hiswin.ncols, hiswin.beginy, hiswin.beginx);
   scrollok(hiswin.win, FALSE);
   clearok(hiswin.win, FALSE);
   wclear(hiswin.win);

   wmove(mywin.win, mywin.nlines - 1, 0);
   wnoutrefresh(mywin.win);
   wnoutrefresh(centerlinewin.win);
   wnoutrefresh(hiswin.win);
   doupdate();

}

BOOLEAN
connection(char *host, int port, int *fdsocket)
{

   struct sockaddr_in    temp;
   struct hostent      * hent;
   BOOLEAN               status = ISTRUE;


   temp.sin_family    = AF_INET;
   temp.sin_port      = htons(port);
   hent               = gethostbyname(host);
      
   if (hent == 0)
      status = ISFALSE;
   else 
   {
      bcopy(hent->h_addr, &temp.sin_addr, hent->h_length);

      if ((*fdsocket = socket(PF_INET, SOCK_STREAM, 0)) < 0)
         status = ISFALSE;
      else         
         if ((connect(*fdsocket, (struct sockaddr *) &temp, sizeof(temp))) < 0)
            status = ISFALSE;
   }
      
return status;
}

BOOLEAN
listening(int *serversocket, int port, int queue) 
{

   struct sockaddr_in    temp;
   BOOLEAN               status = ISTRUE;
   
   
   bzero((char *) &temp, sizeof(temp));

   temp.sin_addr.s_addr   = htonl(INADDR_ANY);
   temp.sin_port          = htons(port);
   temp.sin_family        = AF_INET;
      
   if ((*serversocket = socket(PF_INET ,SOCK_STREAM, 0)) < 0)
      status = ISFALSE;
   else {   
   
      if (bind(*serversocket, (struct sockaddr *)&temp, sizeof(temp)) < 0)
         status = ISFALSE;
      else 
         if (listen(*serversocket, queue) < 0)
            status = ISFALSE;

   }  
   
return status;
}

BOOLEAN
checkPort(const int MINPORTx, const int MAXPORTy, int port)
{ 
    return ((port >= MINPORTx && port <= MAXPORTy) ? ISTRUE : ISFALSE); 
}

/*
  EOF
 */  
