
/*

  0x333stalk_server.c
  
     stalk application was born with target to allow two users talking in encrypted mode.
     his job is that listening any connection, only one connection is accepted...
     the server wait connection and when accept, try exchange public keys...
     and link your output with client, using RSA function for encrypt and decrypt data.
     every time are generated public and private keys, use and getta.
  
     [note]

        working is that usually:
  
          A) sender encrypt msg with public key of adressee <exchange step> then adressee decrypt msg with his private key
          B) adressee encrypt msg with public key of sender <exchange step> then sender decrypt msg with his private key
        
     [/note]      
     
     any configuration values... version banner and etc are in header file config.h
  
     here an example without any parameters:
  
     nsn@creature:~/0x333stalk$ ./srv
     [*] Generating RSA KEY 2048 BITS E: 65537;
     .....................+++
     ............................................+++
     *** WAITING INCOMING CONNECTIONS ***
     *** A connection is accepted remote host: 127.0.0.1
     *** Pubkey RECEVIED (if is incorrect target host will not able decrypt any msg)
     *** KEYS EXCHANGED!
     *** Insert your nickname (max 25 chars): nsn
     *** Waiting client while choose a nickname and give us the OK for talk
     *** Connection Estabelished with c0wboy, now you are talking with him ***
  
     I didn't want make a server multi chat, because if you need talk with more ppl
     you can exchange a session key in a channel like this what 0x333stalk_server open
     and then later use the key exchanged for conversation trough other applications.

     any bugs, ideas send mail to nsn@0x333.org
     
  coded by nsn / 0x333 Outsiders Security Labs / www.0x333.org

*/

#include "config.h"


BOOLEAN           FRIENDLOGGED               = ISFALSE;
int               DEFAULTSERVERPORT          = 0x8235; //can be changed with option -p


void
usage (char *prg)
{

   fprintf (stderr, " [~] %s [~]\n", SERVERBANNER);
   fprintf (stderr, " [~] coded by %s of %s %s [~]\n\n", AUTHOR, TEAM, URL);
   fprintf (stderr, " Usage: %s [ -p port ] [ -h ]\n\n", prg);
   fprintf (stderr, " \t-p :\t port where go in listening (default: %d)\n", DEFAULTSERVERPORT);
   fprintf (stderr, " \t-h :\t display this help\n\n");

   exit(EXITCODE);
   
}

int
main (int argc, char *argv[])
{

   RSA           * myserverkey;
   infofriend      myfriend;
   
   int           * serversocket = malloc(sizeof(int)),
                   c;
   char            datastring[BITSENC], 
                   mynickname[MAXLENNICK];
   

   while (( c = getopt (argc,argv,"h:p:")) != EOF)
   {
      switch(c)
      {
         case 'p' : DEFAULTSERVERPORT = atoi(optarg);break;
         case 'h' : usage(argv[0]);break;
         default  : usage(argv[0]);
      }    
   }
   
   if (checkPort(MINPORT, MAXPORT, DEFAULTSERVERPORT)) {

      signal(SIGINT, myexit);   //catching interrupts
      
      printf("[*] Generating RSA KEYS %d BITS E: %d;\n", BITSENC, COPRIMO);
      myserverkey = generateKEYS(BITSENC, COPRIMO, NAMEFILEPUBKEY_SERVER);
     
      if (listening(serversocket, DEFAULTSERVERPORT, 1)) {

         myfriend.len_sockaddr = sizeof(myfriend.rem_addr);
            
         while(1) {

            printf("*** WAITING INCOMING CONNECTIONS ***\n");
            
            myfriend.socketfriend = accept(*serversocket, (struct sockaddr *)&myfriend.rem_addr, &myfriend.len_sockaddr);
            
            if (myfriend.socketfriend < 0)
               error("Something wrong... lost connection...");
               
            if (FRIENDLOGGED)
               close(myfriend.socketfriend);               
            else {
               
               FRIENDLOGGED = ISTRUE;
               
               printf("*** A connection is accepted remote host: %s\n", inet_ntoa(myfriend.rem_addr.sin_addr));
               writesocket(myfriend.socketfriend, NORMALTEXT,"%s\n", SERVERBANNER);  
   
               memset(datastring, 0, BITSENC); 
               read(myfriend.socketfriend, datastring, BITSENC - 1);

               /* PROTO STEP 1 */
               
               if (!memcmp(datastring, "EXCHANGEKEYS", strlen("EXCHANGEKEYS"))) {
                  if (exchangeKeys(myfriend.socketfriend, &(myfriend.pubkeyfriend), NAMEFILEPUBKEY_SERVER, NAMETEMPFILEPUBKEY_CLIENT))
                     printf("*** KEYS EXCHANGED!\n");
                  else {
                     writesocket(myfriend.socketfriend, CLOSESOCKET, "Something wrong... while exchanging pubkeys");
                     continue;
                  }   
               }  
               else {
                  writesocket(myfriend.socketfriend, CLOSESOCKET, "Protocol mismatch PROTO STEP 1");
                  continue;
               }   
                  
               /* PROTO STEP 2 */
               
               printf("*** Insert your nickname (max %d chars): ", MAXLENNICK);
               fgets(mynickname, MAXLENNICK, stdin);
               mynickname[strlen(mynickname) - 1] = '\0';
               
               printf("*** Waiting client while choose a nickname and give us the OK for talk\n");
               writesocket(myfriend.socketfriend, NORMALTEXT, "TALK OK?\n");
               memset(datastring, 0, BITSENC); 
               read(myfriend.socketfriend, datastring, BITSENC - 1);
               
               /* PROTO STEP 3 */
                  
               if (!memcmp(datastring, "TALK OK!", strlen("TALK OK!")))
                  stalk(myfriend.socketfriend, myfriend.pubkeyfriend, myserverkey, mynickname);
               else
                  writesocket(myfriend.socketfriend, CLOSESOCKET, "Protocol mismatch PROTO STEP 3");
                  
            }      

            FRIENDLOGGED = ISFALSE;

         }
         
      }
      else
         error("Failed Binding Port %d check your permission", DEFAULTSERVERPORT);
      
   }
   else
      error("Port have to be between %d-%d", MINPORT, MAXPORT);
      

  free(serversocket);
  free(myserverkey);
  remove(NAMEFILEPUBKEY_SERVER);
  remove(NAMETEMPFILEPUBKEY_CLIENT);
   
}

/*
  EOF
 */ 
